@@ -1,10 +1,10 @@
 //
-// $Id: ha_sphinx.cc 4421 2013-12-18 13:05:50Z deogar $
+// $Id: ha_sphinx.cc 4798 2014-09-02 11:26:02Z deogar $
 //
 
 //
-// Copyright (c) 2001-2013, Andrew Aksyonoff
-// Copyright (c) 2008-2013, Sphinx Technologies Inc
+// Copyright (c) 2001-2014, Andrew Aksyonoff
+// Copyright (c) 2008-2014, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -154,7 +154,7 @@ void sphUnalignedWrite ( void * pPtr, const T & tVal )
 #define SPHINXSE_MAX_ALLOC			(16*1024*1024)
 #define SPHINXSE_MAX_KEYWORDSTATS	4096
 
-#define SPHINXSE_VERSION			"2.1.4-release"
+#define SPHINXSE_VERSION			"2.2.4-release"
 
 // FIXME? the following is cut-n-paste from sphinx.h and searchd.cpp
 // cut-n-paste is somewhat simpler that adding dependencies however..
@@ -1273,7 +1273,7 @@ CSphSEQuery::CSphSEQuery ( const char * sQuery, int iLength, const char * sIndex
 	, m_fGeoLatitude ( 0.0f )
 	, m_fGeoLongitude ( 0.0f )
 	, m_sComment ( "" )
-	, m_sSelect ( "" )
+	, m_sSelect ( "*" )
 
 	, m_pBuf ( NULL )
 	, m_pCur ( NULL )
@@ -3624,5 +3624,5 @@ mysql_declare_plugin_end;
 #endif // >50100
 
 //
-// $Id: ha_sphinx.cc 4421 2013-12-18 13:05:50Z deogar $
+// $Id: ha_sphinx.cc 4798 2014-09-02 11:26:02Z deogar $
 //
