@@ -1,10 +1,10 @@
 //
-// $Id: indextool.cpp 4236 2013-10-08 11:41:24Z tomat $
+// $Id: indextool.cpp 4796 2014-09-01 19:45:49Z kevg $
 //
 
 //
-// Copyright (c) 2001-2013, Andrew Aksyonoff
-// Copyright (c) 2008-2013, Sphinx Technologies Inc
+// Copyright (c) 2001-2014, Andrew Aksyonoff
+// Copyright (c) 2008-2014, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,13 @@
 #include <time.h>
 
 #if USE_WINDOWS
-#include <io.h> // for setmode and open on windows
+#include <io.h> // for setmode(). open() on windows
+#define sphSeek		_lseeki64
+#else
+#define sphSeek		lseek
 #endif
 
+
 void StripStdin ( const char * sIndexAttrs, const char * sRemoveElements )
 {
 	CSphString sError;
@@ -104,10 +108,6 @@ void CharsetFold ( CSphIndex * pIndex, FILE * fp )
 	CSphVector<BYTE> sBuf1 ( 16384 );
 	CSphVector<BYTE> sBuf2 ( 16384 );
 
-	bool bUtf = pIndex->GetTokenizer()->IsUtf8();
-	if ( !bUtf )
-		sphDie ( "sorry, --fold vs SBCS is not supported just yet" );
-
 	CSphLowercaser tLC = pIndex->GetTokenizer()->GetLowercaser();
 
 #if USE_WINDOWS
@@ -127,8 +127,8 @@ void CharsetFold ( CSphIndex * pIndex, FILE * fp )
 					break;
 
 
-		BYTE * pIn = sBuf1.Begin();
-		BYTE * pInMax = pIn + iBuf1 + iGot;
+		const BYTE * pIn = sBuf1.Begin();
+		const BYTE * pInMax = pIn + iBuf1 + iGot;
 
 		if ( pIn==pInMax && feof(fp) )
 			break;
@@ -177,12 +177,6 @@ void CharsetFold ( CSphIndex * pIndex, FILE * fp )
 
 //////////////////////////////////////////////////////////////////////////
 
-#if USE_WINDOWS
-#define sphSeek		_lseeki64
-#else
-#define sphSeek		lseek
-#endif
-
 bool FixupFiles ( const CSphVector<CSphString> & dFiles, CSphString & sError )
 {
 	ARRAY_FOREACH ( i, dFiles )
@@ -476,7 +470,7 @@ bool BuildIDF ( const CSphString & sFilename, const CSphVector<CSphString> & dFi
 					if ( !bSkipUnique || iDocs>1 )
 					{
 						IDFWord_t & tEntry = dEntries.Add ();
-						tEntry.m_uWordID = sphFNV64 ( (BYTE*)sWord );
+						tEntry.m_uWordID = sphFNV64 ( sWord );
 						tEntry.m_iDocs = iDocs;
 						iTotalWords++;
 					} else
@@ -819,6 +813,7 @@ int main ( int argc, char ** argv )
 	bool bSkipUnique = false;
 	CSphString sDumpDict;
 	bool bQuiet = false;
+	bool bRotate = false;
 
 	enum
 	{
@@ -855,6 +850,7 @@ int main ( int argc, char ** argv )
 		OPT1 ( "--dumpconfig" )		{ eCommand = CMD_DUMPCONFIG; sDumpHeader = argv[++i]; }
 		OPT1 ( "--dumpdocids" )		{ eCommand = CMD_DUMPDOCIDS; sIndex = argv[++i]; }
 		OPT1 ( "--check" )			{ eCommand = CMD_CHECK; sIndex = argv[++i]; sphSetDebugCheck(); }
+		OPT1 ( "--rotate" )			{ bRotate = true; }
 		OPT1 ( "--htmlstrip" )		{ eCommand = CMD_STRIP; sIndex = argv[++i]; }
 		OPT1 ( "--build-infixes" )	{ eCommand = CMD_BUILDINFIXES; sIndex = argv[++i]; }
 		OPT1 ( "--build-skips" )	{ eCommand = CMD_BUILDSKIPS; sIndex = argv[++i]; }
@@ -952,22 +948,19 @@ int main ( int argc, char ** argv )
 	// load proper config
 	//////////////////////
 
+	CSphString sError;
+	if ( !sphInitCharsetAliasTable ( sError ) )
+		sphDie ( "failed to init charset alias table: %s", sError.cstr() );
+
 	CSphConfigParser cp;
 	CSphConfig & hConf = cp.m_tConf;
 	for ( ;; )
 	{
-		if ( ( eCommand==CMD_DUMPHEADER || eCommand==CMD_DUMPCONFIG ) && sDumpHeader.Ends ( ".sph" ) )
-			break;
-
 		if ( eCommand==CMD_BUILDIDF || eCommand==CMD_MERGEIDF )
 			break;
 
-		if ( eCommand==CMD_DUMPDICT )
-		{
-			if ( sDumpDict.Ends ( ".spi" ) )
-				break;
-			sIndex = sDumpDict;
-		}
+		if ( eCommand==CMD_DUMPDICT && !sDumpDict.Ends ( ".spi" ) )
+				sIndex = sDumpDict;
 
 		sphLoadConfig ( sOptConfig, bQuiet, cp );
 		break;
@@ -976,6 +969,16 @@ int main ( int argc, char ** argv )
 	///////////
 	// action!
 	///////////
+	int iMvaDefault = 1048576;
+	if ( hConf.Exists ( "searchd" ) && hConf["searchd"].Exists ( "searchd" ) )
+	{
+		const CSphConfigSection & hSearchd = hConf["searchd"]["searchd"];
+		iMvaDefault = hSearchd.GetSize ( "mva_updates_pool", iMvaDefault );
+	}
+	const char * sArenaError = sphArenaInit ( iMvaDefault );
+	if ( sArenaError )
+		sphWarning ( "process shared mutex unsupported, persist MVA disabled ( %s )", sArenaError );
+
 
 	if ( eCommand==CMD_CHECKCONFIG )
 	{
@@ -989,11 +992,11 @@ int main ( int argc, char ** argv )
 		while ( hIndexes.IterateNext() )
 		{
 			const CSphConfigSection & tIndex = hIndexes.IterateGet();
-			const CSphString * pPath = tIndex ( "path" );
+			const CSphVariant * pPath = tIndex ( "path" );
 			if ( !pPath )
 				continue;
 
-			const CSphString * pType = tIndex ( "type" );
+			const CSphVariant * pType = tIndex ( "type" );
 			if ( pType && ( *pType=="rt" || *pType=="distributed" ) )
 				continue;
 
@@ -1023,6 +1026,8 @@ int main ( int argc, char ** argv )
 		}
 	}
 
+	// configure common settings (as of time of this writing, AOT and RLP setup)
+	sphConfigureCommon ( hConf );
 
 	// common part for several commands, check and preload index
 	CSphIndex * pIndex = NULL;
@@ -1056,15 +1061,21 @@ int main ( int argc, char ** argv )
 				pIndex = sphCreateIndexRT ( tSchema, sIndex.cstr(), 32*1024*1024, hConf["index"][sIndex]["path"].cstr(), bDictKeywords );
 		} else
 		{
-			pIndex = sphCreateIndexPhrase ( sIndex.cstr(), hConf["index"][sIndex]["path"].cstr() );
+			const char * sPath = hConf["index"][sIndex]["path"].cstr();
+			CSphStringBuilder tPath;
+			if ( bRotate )
+			{
+				tPath.Appendf ( "%s.tmp", sPath );
+				sPath = tPath.cstr();
+			}
+			pIndex = sphCreateIndexPhrase ( sIndex.cstr(), sPath );
 		}
 
 		if ( !pIndex )
 			sphDie ( "index '%s': failed to create (%s)", sIndex.cstr(), sError.cstr() );
 
-		// don't need any long load operations
-		// but not for dict=keywords + infix
-		pIndex->SetWordlistPreload ( bDictKeywords );
+		if ( eCommand==CMD_CHECK )
+			pIndex->SetDebugCheck();
 
 		CSphString sWarn;
 		if ( !pIndex->Prealloc ( false, bStripPath, sWarn ) )
@@ -1080,7 +1091,7 @@ int main ( int argc, char ** argv )
 		{
 			CSphIndexSettings tSettings = pIndex->GetSettings();
 
-			const CSphString & sValue = hConf["index"][sIndex]["hitless_words"];
+			const CSphString & sValue = hConf["index"][sIndex]["hitless_words"].strval();
 			if ( sValue=="all" )
 			{
 				tSettings.m_eHitless = SPH_HITLESS_ALL;
@@ -1096,6 +1107,9 @@ int main ( int argc, char ** argv )
 		break;
 	}
 
+	int iCheckErrno = 0;
+	CSphString sNewIndex;
+
 	// do the dew
 	switch ( eCommand )
 	{
@@ -1146,8 +1160,6 @@ int main ( int argc, char ** argv )
 				if ( !pIndex )
 					sphDie ( "index '%s': failed to create (%s)", sIndex.cstr(), sError.cstr() );
 
-				pIndex->SetWordlistPreload ( true );
-
 				CSphString sWarn;
 				if ( !pIndex->Prealloc ( false, bStripPath, sWarn ) )
 					sphDie ( "index '%s': prealloc failed: %s\n", sIndex.cstr(), pIndex->GetLastError().cstr() );
@@ -1165,7 +1177,17 @@ int main ( int argc, char ** argv )
 
 		case CMD_CHECK:
 			fprintf ( stdout, "checking index '%s'...\n", sIndex.cstr() );
-			return pIndex->DebugCheck ( stdout );
+			iCheckErrno = pIndex->DebugCheck ( stdout );
+			if ( iCheckErrno )
+				return iCheckErrno;
+			if ( bRotate )
+			{
+				pIndex->Dealloc();
+				sNewIndex.SetSprintf ( "%s.new", hConf["index"][sIndex]["path"].cstr() );
+				if ( !pIndex->Rename ( sNewIndex.cstr() ) )
+					sphDie ( "index '%s': rotate failed: %s\n", sIndex.cstr(), pIndex->GetLastError().cstr() );
+			}
+			return 0;
 
 		case CMD_STRIP:
 			{
@@ -1212,7 +1234,7 @@ int main ( int argc, char ** argv )
 		{
 			CSphString sError;
 			if ( !BuildIDF ( sOut, dFiles, sError, bSkipUnique ) )
-				fprintf ( stdout, "ERROR: %s\n", sError.cstr() );
+				sphDie ( "ERROR: %s\n", sError.cstr() );
 			break;
 		}
 
@@ -1220,7 +1242,7 @@ int main ( int argc, char ** argv )
 		{
 			CSphString sError;
 			if ( !MergeIDF ( sOut, dFiles, sError, bSkipUnique ) )
-				fprintf ( stdout, "ERROR: %s\n", sError.cstr() );
+				sphDie ( "ERROR: %s\n", sError.cstr() );
 			break;
 		}
 
@@ -1247,5 +1269,5 @@ int main ( int argc, char ** argv )
 }
 
 //
-// $Id: indextool.cpp 4236 2013-10-08 11:41:24Z tomat $
+// $Id: indextool.cpp 4796 2014-09-01 19:45:49Z kevg $
 //
