@@ -1,10 +1,10 @@
 //
-// $Id: spelldump.cpp 3701 2013-02-20 18:10:18Z deogar $
+// $Id: spelldump.cpp 4522 2014-01-30 11:00:18Z tomat $
 //
 
 //
-// Copyright (c) 2001-2013, Andrew Aksyonoff
-// Copyright (c) 2008-2013, Sphinx Technologies Inc
+// Copyright (c) 2001-2014, Andrew Aksyonoff
+// Copyright (c) 2008-2014, Sphinx Technologies Inc
 // All rights reserved
 //
 // This program is free software; you can redistribute it and/or modify
@@ -296,18 +296,14 @@ bool CISpellAffixRule::CheckSuffix ( const CSphString & sWord ) const
 		if ( m_sCondition.cstr()[iCondI]=='.' )
 		{
 			--iCondI;
-		}
-		else if ( m_sCondition.cstr()[iCondI]!=']' )
+		} else if ( m_sCondition.cstr()[iCondI]!=']' )
 		{
 			if ( m_sCondition.cstr()[iCondI]!=sWord.cstr()[i] )
 				return false;
-
 			--iCondI;
-
 		} else
 		{
 			int iRangeStart = -1;
-
 			for ( int j=iCondI; j>=0 && iRangeStart==-1; --j )
 				if ( m_sCondition.cstr()[j]=='[' )
 					iRangeStart = j;
@@ -318,12 +314,10 @@ bool CISpellAffixRule::CheckSuffix ( const CSphString & sWord ) const
 			{
 				if ( !IsInSet ( sWord.cstr () [i], m_sCondition.SubString ( iRangeStart + 1, iCondI - iRangeStart - 1 ).cstr () ) )
 					return false;
-
 				iCondI = iRangeStart - 1;
 			}
 		}
 	}
-
 	return true;
 }
 
@@ -1195,5 +1189,5 @@ int main ( int iArgs, char ** dArgs )
 }
 
 //
-// $Id: spelldump.cpp 3701 2013-02-20 18:10:18Z deogar $
+// $Id: spelldump.cpp 4522 2014-01-30 11:00:18Z tomat $
 //
